<?php

namespace Service;

use \Util\Exception;

class Promocao {
    protected $entityManager;
	protected $util;
    protected $precoAPI;
    protected $promocaoAPI;

    public function __construct(
		\Doctrine\ORM\EntityManager $entityManager,
		\Util\Util $util,
        \Odhen\API\Service\Preco $precoAPI
	){
		$this->entityManager      = $entityManager;
		$this->util               = $util;
        $this->precoAPI           = $precoAPI;
	}

    //TIRADOS DE Pedido.php
    public function formataPromocaoCombinada($cdfilial, $cdcliente, $cdconsumidor, $cdloja, $produtos, $CDPRODPROMOCAO, $nrVendaRest, $ordemImp, $isCombo, $IDTIPCOBRA, $IDIMPPRODUTO, $produtoCombo){
		$composicao = array();
		$treatedProducts = array();

		self::trataProdutosPromocao($cdfilial, $cdcliente, $cdconsumidor, $cdloja, $composicao, $treatedProducts, $produtos, $CDPRODPROMOCAO, $ordemImp, $isCombo, $IDIMPPRODUTO, $produtoCombo);

		// rotina específica do Madero
		$this->util->validaDescontoDiferenciado($cdfilial, $CDPRODPROMOCAO, $composicao, 'VRDESCCOMVEN');

		// Trata os preços de produtos combinados.
		if ($isCombo){
			// Verifica o preço mais caro.
			$maxPrice = 0;
			if ($IDTIPCOBRA === 'C'){
				foreach ($composicao as $comboProd){
					if ($comboProd['VRPRECCOMVEN'] > $maxPrice){
						$maxPrice = $comboProd['VRPRECCOMVEN'];
					}
				}
			}

			// Implementa os ajustes de preço e quantidade.
			foreach ($composicao as &$comboProd){
				if ($IDTIPCOBRA == 'C'){
					$comboProd['VRPRECCOMVEN'] = $maxPrice;
				}
			}
		}

        // Campanha promocional.
        if ($IDIMPPRODUTO == '2'){ // Somente produtos filho que são cobrados.
            // Hora do pedido.
            if (isset($produtos[0]['DATETIME'])){
                $time = $produtos[0]['DATETIME']; // Hora do pedido quando tiver sendo recalculado.
            }
            else {
                $time = new \DateTime();
            }
            $time = $time->format('Hi');
            // Busca dados da campanha.
            $campanha = $this->campanhaPromocional($cdcliente, $cdconsumidor, $produtos, $time);
            // Define os produtos que serão modificados.
            if (!empty($campanha)){
                if ($campanha['IDAPLICADESACR'] == '1'){
                    self::aplicaDescontoCampanha($composicao, $campanha, $campanha['CDPRODPRIN']);
                }
                else if ($campanha['IDAPLICADESACR'] == '2'){
                    self::aplicaDescontoCampanha($composicao, $campanha, $campanha['CDPRODCOMB']);
                }
                else if ($campanha['IDAPLICADESACR'] == '3'){
                    self::aplicaDescontoCampanha($composicao, $campanha, $campanha['CDPRODCOMB2']);
                }
                else if ($campanha['IDAPLICADESACR'] == '4'){ // Aplica o valor do desconto em todos os itens.
                    self::rateiaDescontoCampanha($composicao, $campanha);
                }
            }
        }

        self::aplicaDescontoVoucher($composicao);

		return $composicao;
	}

    private function trataProdutosPromocao($cdfilial, $cdcliente, $cdconsumidor, $cdloja, &$composicao, &$treatedProducts, &$produtos, $CDPRODPROMOCAO, $ordemImp, $isCombo, $IDIMPPRODUTO, $produtoCombo) {
        foreach ($produtos as $produto){
			$precoPromocao = $this->precoAPI->buscaPreco($cdfilial, $cdcliente, $produto['CDPRODUTO'], $cdloja, $cdconsumidor);

			$precoTotal = $precoPromocao["PRECO"] + $precoPromocao["PRECOCLIE"] - $precoPromocao["DESC"] + $precoPromocao["ACRE"];

            // Busca voucher.
            if (!empty($produto['VOUCHER'])){
                $params = array(
                    'CDCUPOMDESCFOS' => $produto['VOUCHER']['CDCUPOMDESCFOS']
                );
                $voucherData = $this->entityManager->getConnection()->fetchAssoc("SQL_VOUCHER_DATA", $params);
            } else {
                $voucherData = null;
            }
			$item = $produto;//array();
			if(empty($item['posicao']) && !empty($produtoCombo['posicao'])){
				$item['posicao'] = $produto['posicao'];
			}
			if(empty($item['NRLUGARMESA']) && !empty($produtoCombo['NRLUGARMESA'])){
				$item['NRLUGARMESA'] = $produto['NRLUGARMESA'];
			}
            $item["CDPRODUTOPAI"]   = $CDPRODPROMOCAO;
			$item["CDPRODPROMOCAO"] = $CDPRODPROMOCAO;
            $item["VRPRECCOMVEN"]   = floatval($precoPromocao["PRECO"]) ? floatval($precoPromocao["PRECO"]) : 0.0;
            $item["VRPRECCLCOMVEN"] = floatval($precoPromocao["PRECOCLIE"]) ? floatval($precoPromocao["PRECOCLIE"]) : 0.0;
            $item["QTPRODCOMVEN"]   = isset($produto['QTPRODCOMVEN']) ? $produto['QTPRODCOMVEN'] : 1;
            $item["ORDEMIMP"]       = $ordemImp;
            $item["DSOBSPEDDIG"]    = isset($produto['DSOCORR_CUSTOM'])?$produto['DSOCORR_CUSTOM']:$produto['CUSTOMOBS'];
            $item["IMPRESSORA"]     = isset($produto['PRINTER'])?$produto['PRINTER']:null;
            $item['VRDESCCOMVEN']   = floatval(bcmul(strval($precoPromocao['DESC']), strval($item['QTPRODCOMVEN']), 2));
            $item['VRACRCOMVEN']    = floatval(bcmul(strval($precoPromocao['ACRE']), strval($item['QTPRODCOMVEN']), 2));
            $item['PRECOREAL']      = $precoTotal;
            $item['VOUCHER']        = $voucherData;
            $item['IMPRIMESIM']     = $produto['IDIMPGRPROMO'] == 'S';
            $item['IDIMPPRODUTO_pai']= $produtoCombo['IDIMPPRODUTO'];
            $item['ORIGINALQUANT']  = $item['ORIGINALQUANT'];
            $item['COMBOQUANT']     = isset($produtoCombo['COMBOQUANT']) ? $produtoCombo['COMBOQUANT'] : null;
			$IDIMPGRPROMOtemp = $produto['IDIMPGRPROMO'];
			if($IDIMPPRODUTO == '2'){ // se o combo for imprime não, não valida o parametro dos itens do combo
				$IDIMPGRPROMOtemp = 'N';
			}
            if (!$isCombo && $IDIMPGRPROMOtemp != 'S'){
                $descontoPromocao = $this->precoAPI->promocaoAPI->calculaDesconto($cdfilial, $CDPRODPROMOCAO, $produto['CDPRODUTO'], $precoTotal, $treatedProducts);
                if ($descontoPromocao < 0) $item['VRDESCCOMVEN'] -= floatval(bcmul(strval($descontoPromocao), strval($item['QTPRODCOMVEN']), 2));
                else $item['VRACRCOMVEN'] += $descontoPromocao;
            }

            array_push($composicao, $item);
			array_push($treatedProducts, $produto['CDPRODUTO']);
		}
    }

    private function aplicaDescontoVoucher(&$composicao) {
        // Aplica descontos do voucher.
        foreach ($composicao as &$produto){
            if ($produto['VOUCHER']){
                $total = floatval(bcsub(strval($produto['QTPRODCOMVEN'] * ($produto['VRPRECCOMVEN'] + $produto['VRPRECCLCOMVEN']) + $produto['VRACRCOMVEN']), strval($produto['VRDESCCOMVEN']), 2));
                if ($produto['VOUCHER']['IDTIPODESC'] === "P"){
                    $voucherDiscount = bcmul(strval($total), strval($produto['VOUCHER']['VRDESCCUPOM']/100), 2);
                }
                else {
                    $voucherDiscount = $produto['VOUCHER']['VRDESCCUPOM'];
                }
                $produto['VRDESCCOMVEN'] += floatval($voucherDiscount);

                $precoReal = floatval(bcadd(strval($produto['QTPRODCOMVEN'] * ($produto['VRPRECCOMVEN'] + $produto['VRPRECCLCOMVEN'])), strval($produto['VRACRCOMVEN']), 2));
                if ($produto['VRDESCCOMVEN'] >= $precoReal){
                    $produto['VRDESCCOMVEN'] = floatval(bcsub(strval($precoReal), strval($produto['QTPRODCOMVEN'] * 0.01), 2));
                }
                $produto['VOUCHER'] = $produto['VOUCHER']['CDCUPOMDESCFOS'];
            }
        }
    }

    private function aplicaDescontoCampanha(&$composicao, $campanha, $CDPRODUTO){
        foreach ($composicao as &$produto){
            if ($produto['CDPRODUTO'] == $CDPRODUTO){
                if ($campanha['IDPERCVALOR'] == 'V'){
                    $valor = floatval($campanha['VRDESCACRE']);
                }
                else {
                    $valor = $produto['PRECOREAL'] * floatval($campanha['VRDESCACRE']) / 100;
                }

                if ($campanha['IDDESCACRE'] == 'D'){
                    $produto['VRDESCCOMVEN'] = floatval(bcmul(str_replace(',','.',strval($valor)), '1', '2'));
                }
                else {
                    $produto['VRACRCOMVEN'] = floatval(bcmul(str_replace(',','.',strval($valor)), '1', '2'));
                }
                break;
            };
        }
    }

    private function rateiaDescontoCampanha(&$composicao, $campanha){
        // Preco total de TOTOS os produtos, considerando descontos e acrescimos da ITEMPRECODIA.
        $totalPrice = array_reduce($composicao, function($total, $produto){
            return $total + $produto['PRECOREAL'];
        }, 0);
        // Preco final de TODOS os produtos, considerando o desconto ou acrescimo da campanha.
        if ($campanha['IDDESCACRE'] == 'D'){
            $modo = 'VRDESCCOMVEN';
            if ($campanha['IDPERCVALOR'] == 'V'){
                $finalPrice = $totalPrice - floatval($campanha['VRDESCACRE']);
            }
            else {
                $finalPrice = $totalPrice * (1 - floatval($campanha['VRDESCACRE']) / 100);
            }
        }
        else {
            $modo = 'VRACRCOMVEN';
            if ($campanha['IDPERCVALOR'] == 'V'){
                $finalPrice = $totalPrice + floatval($campanha['VRDESCACRE']);
            }
            else {
                $finalPrice = $totalPrice * (1 + floatval($campanha['VRDESCACRE']) / 100);
            }
        }
        // Aplica o desconto/acrescimo parcial nos produtos.
        foreach ($composicao as &$produto){
            if ($campanha['IDPERCVALOR'] == 'V'){
                $valor = ($produto['PRECOREAL'] / $totalPrice) * floatval($campanha['VRDESCACRE']);
            }
            else {
                $valor = $produto['PRECOREAL'] * floatval($campanha['VRDESCACRE']) / 100;
            }
            $produto[$modo] = floatval(bcmul(str_replace(',','.',strval($valor)), '1', '2'));
        }
        // Total do desconto/acrescimo implementado.
        $totalDescAcre = 0;
        foreach ($composicao as &$produto){
            $totalDescAcre += $produto[$modo];
        }
        // Rateia a diferença entre os descontos/acrescimos, caso exista.
        $diferenca = ($totalPrice - $finalPrice) - $totalDescAcre;
        if ($diferenca > 0.01){
            $qtdRateio = intval($diferenca/0.01);
            $i = 0;
            while ($qtdRateio > 0){
                foreach ($composicao as &$produto){
                    $totalProduto = round($produto['PRECOREAL'] + $produto['VRACRCOMVEN'] - $produto['VRDESCCOMVEN'], 2);
                    if ($produto[$modo] >= 0.01 && $totalProduto > 0.01){
                        $produto[$modo] += 0.01;
                        $qtdRateio--;
                    }
                    if ($qtdRateio == 0) break;
                }

                $i++;
                if ($i == 1000) throw new \Exception("Erro no rateio do desconto.");
            }
        }
    }

    public function campanhaPromocional($CDCLIENTE, $CDCONSUMIDOR, $PRODUTOS, $HORA){
        // Busca tipo do consumidor.
        $params = array(
            'CDCLIENTE' => $CDCLIENTE,
            'CDCONSUMIDOR' => $CDCONSUMIDOR
        );
        $CDTIPOCONS = $this->entityManager->getConnection()->fetchAssoc("TIPO_CONSUMIDOR", $params);
        $CDTIPOCONS = !empty($CDTIPOCONS) ? $CDTIPOCONS['CDTIPOCONS'] : null;
        // Constrói parâmetro para a query.
        $productString = '_';
        foreach ($PRODUTOS as $product){
            $productString .= $product['CDPRODUTO'].'_';
        }
        // Busca dados da campanha.
        $params = array(
            'CDCLIENTE' => $CDCLIENTE,
            'CDTIPOCONS' => $CDTIPOCONS,
            'HORA' => $HORA,
            'STRPRODUCTS' => $productString
        );
        $campanha = $this->entityManager->getConnection()->fetchAssoc("BUSCA_DADOS_CAMPANHA", $params);

        if ($campanha === false) $campanha = array();
        return $campanha;
    }

    
}